unit RunButton2;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Buttons, DsgnIntf;

type
  TRunEvent = procedure(Sender: TObject; Command: String;
    HProcess: THandle) of object;
  TRanEvent = procedure(Sender: TObject; Command: String) of object;

  TRunButton = class(TSpeedButton)
  private
    FCommand: String;
    FOnRun: TRunEvent;
    FOnRan: TRanEvent;
    procedure DoThreadEnd(Sender: TObject);
  protected
    procedure SetCommand(Cmd: String); virtual;
    procedure Click; override;
  public
    function Run: Boolean; virtual;
  published
    property Command: String read FCommand write SetCommand;
    property OnRun: TRunEvent read FOnRun write FOnRun;
    property OnRan: TRanEvent read FOnRan write FOnRan;
  end;

  TWaitThread = class(TThread)
  private
    HProcess: THandle;
  public
    constructor Create(Process: THandle);
    procedure Execute; override;
  end;

  TCommandProperty = class(TStringProperty)
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  TRunButtonEditor = class(TComponentEditor)
    procedure Edit; override;
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

procedure Register;

implementation

{ Link in component bitmap }
{$R RunBtn.DCR}

procedure TRunButton.DoThreadEnd(Sender: TObject);
begin
  if Assigned(FOnRan) then
    FOnRan(Self, Command);
end;

procedure TRunButton.Click;
begin
  Run;
  inherited Click;
end;

function TRunButton.Run: Boolean;
var
  SI: TStartupInfo;
  PI: TProcessInformation;
begin
  GetStartupInfo(SI);
  Result := CreateProcess(nil, PChar(FCommand),
    nil, nil, False, 0, nil, nil, SI, PI);
  if Result then
  begin
    with TWaitThread.Create(PI.HProcess) do
    begin
      FreeOnTerminate := True;
      OnTerminate := DoThreadEnd;
      Resume
    end;
    if Assigned(FOnRun) then
      FOnRun(Self, FCommand, PI.HProcess);
  end;
end;

procedure TRunButton.SetCommand(Cmd: String);
begin
  if Length(Cmd) <= Max_Path then
    FCommand := Cmd;
end;

constructor TWaitThread.Create(Process: THandle);
begin
  inherited Create(True);
  HProcess := Process;
end;

procedure TWaitThread.Execute;
begin
  WaitForSingleObject(HProcess, Infinite);
end;

procedure TCommandProperty.Edit;
begin
  with TOpenDialog.Create(Application) do
    try
      FileName := Value;
      Filter := 'Programs (*.exe)|*.exe';
      if Execute then
        Value := FileName;
    finally
      Free;
    end;
end;

function TCommandProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paMultiSelect];
end;

procedure TRunButtonEditor.Edit;
begin
  ExecuteVerb(1);
end;

procedure TRunButtonEditor.ExecuteVerb(Index: Integer);
begin
  case Index of
    0: ShowMessage('Crafted by the hands of Brian');
    1: ShowMessage('A second component editor');
    2: ShowMessage('And a third one');
  end;
end;

function TRunButtonEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := 'About box';
    1: Result := 'Second possibility';
    2: Result := 'Last editor';
  end;
end;

function TRunButtonEditor.GetVerbCount: Integer;
begin
  Result := 3;
end;

procedure Register;
begin
  RegisterComponents('New Ones', [TRunButton]);
  RegisterPropertyEditor(TypeInfo(String), TRunButton,
    'Command', TCommandProperty);
  RegisterComponentEditor(TRunButton, TRunButtonEditor);
end;

end.
